from krate import settings
from django.utils.importlib import import_module

__all__ = ('ratehandler',)

module_name = ".".join(settings.KRATE_RATE_HANDLER.split(".")[0:-1])
class_name = settings.KRATE_RATE_HANDLER.split(".")[-1]
ratehandler = getattr(import_module(module_name), class_name)()



from django import forms
from .settings import *
from django.forms.widgets import RadioSelect

KRATE_CHOICES = [ (x,str(x)) for x in KRATE_DEFAULT_CHOICES ]

class KRateForm(forms.Form):
    value = forms.ChoiceField(widget=forms.widgets.RadioSelect,
                              choices=KRATE_CHOICES)



from django.conf import settings

KRATE_RATE_HANDLER = getattr(settings, 'KRATE_RATE_HANDLER', 'krate.ratehandlers.noratehandler.NoRateHandler')
KRATE_DEFAULT_CHOICES = getattr(settings, 'KRATE_DEFAULT_CHOICES', [1, 2, 3, 4, 5])



from .ratehandlerbase import RateHandlerBase

class NoRateHandler(RateHandlerBase):
    def rate_object(self, request, obj, rate, **kwargs):
        return ('created', 0)

    def get_object_rate(self, obj, **kwargs):
        return 0

    def get_request_object_rate(self, user, obj, **kwargs):
        return 0

    def get_user_object_rate(self, user, obj, **kwargs):
        return 0



import unittest

from django.http import HttpRequest
from django.contrib.auth.models import User
from django.template import Template, Context, TemplateSyntaxError

from krate.ratehandlers.dbratehandler import DBRateHandler
from krate.ratehandlers.dbratehandler.models import *

from testing.models import TestModel


class KRateTemplateTagsTest(unittest.TestCase):
    def setUp(self):
        self.handler = DBRateHandler()
        self.request1 = HttpRequest()
        self.request1.META = {
            'CONTENT_LENGTH': '',
            'CONTENT_TYPE': 'text/html',
            'HTTP_ACCEPT_ENCODING': 'utf-8',
            'HTTP_ACCEPT_LANGUAGE': 'es',
            'HTTP_HOST': 'testhost',
            'HTTP_REFERER': 'testreferer',
            'HTTP_USER_AGENT': 'test user agent',
            'QUERY_STRING': '/',
            'REMOTE_ADDR': 'testaddress',
            'REMOTE_HOST': 'testhost',
            'REMOTE_USER': 'testuser',
            'REQUEST_METHOD': 'GET',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': '80',
        }
        (self.request1.user, created) = User.objects.get_or_create(username='testuser1')
        self.request1.user.save()

        self.request2 = HttpRequest()
        self.request2.META = {
            'CONTENT_LENGTH': '',
            'CONTENT_TYPE': 'text/html',
            'HTTP_ACCEPT_ENCODING': 'utf-8',
            'HTTP_ACCEPT_LANGUAGE': 'es',
            'HTTP_HOST': 'testhost',
            'HTTP_REFERER': 'testreferer',
            'HTTP_USER_AGENT': 'test user agent',
            'QUERY_STRING': '/',
            'REMOTE_ADDR': 'testaddress',
            'REMOTE_HOST': 'testhost',
            'REMOTE_USER': 'testuser',
            'REQUEST_METHOD': 'GET',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': '80',
        }
        (self.request2.user, created) = User.objects.get_or_create(username='testuser2')
        self.request2.user.save()
        self.test1 = TestModel()
        self.test1.save()
        self.test2 = TestModel()
        self.test2.save()

        self.handler.rate_object(self.request1, self.test1, 10)
        self.handler.rate_object(self.request2, self.test1, 7)

    def test_krate_tag(self):
        template1 = Template('{% load krate %}{% krate obj %}')
        template2 = Template('{% load krate %}{% krate obj default=No%}')

        rendered = template1.render(Context({'obj': self.test1}))
        self.assertEqual(rendered, "8.5")
        rendered = template1.render(Context({'obj': self.test2}))
        self.assertEqual(rendered, "")
        rendered = template2.render(Context({'obj': self.test1}))
        self.assertEqual(rendered, "8.5")
        rendered = template2.render(Context({'obj': self.test2}))
        self.assertEqual(rendered, "No")

    def test_mykrate_tag(self):
        template1 = Template('{% load krate %}{% mykrate obj request_or_user %}')
        template2 = Template('{% load krate %}{% mykrate obj request_or_user default=No%}')

        with self.assertRaises(TemplateSyntaxError):
            Template('{% load krate %}{% mykrate obj request_or_user =%}')

        with self.assertRaises(TemplateSyntaxError):
            Template('{% load krate %}{% mykrate obj request_or_user parameter %}')

        rendered = template1.render(Context({'obj': self.test1, 'request_or_user': self.request1}))
        self.assertEqual(rendered, "10.0")
        rendered = template1.render(Context({'obj': self.test1, 'request_or_user': self.request2}))
        self.assertEqual(rendered, "7.0")
        rendered = template1.render(Context({'obj': self.test2, 'request_or_user': self.request1}))
        self.assertEqual(rendered, "")
        rendered = template1.render(Context({'obj': self.test2, 'request_or_user': self.request2}))
        self.assertEqual(rendered, "")

        rendered = template2.render(Context({'obj': self.test1, 'request_or_user': self.request1}))
        self.assertEqual(rendered, "10.0")
        rendered = template2.render(Context({'obj': self.test1, 'request_or_user': self.request2}))
        self.assertEqual(rendered, "7.0")
        rendered = template2.render(Context({'obj': self.test2, 'request_or_user': self.request1}))
        self.assertEqual(rendered, "No")
        rendered = template2.render(Context({'obj': self.test2, 'request_or_user': self.request2}))
        self.assertEqual(rendered, "No")

        with self.assertRaises(TemplateSyntaxError):
            rendered = template1.render(Context({'obj': self.request1, 'request_or_user': self.test1}))



class RateHandlerBase(object):
    def rate_object(self, request, obj, rate, **kwargs):
        raise NotImplementedError

    def get_object_rate(self, obj, **kwargs):
        raise NotImplementedError

    def get_request_object_rate(self, request, obj, **kwargs):
        raise NotImplementedError

    def get_user_object_rate(self, user, obj, **kwargs):
        raise NotImplementedError



from django import template
from django.http import HttpRequest
from django.contrib.auth.models import User, AnonymousUser
from ..settings import *
from ..ratehandlers import ratehandler

register = template.Library()

class KRateNode(template.Node):
    def __init__(self, obj, **kwargs):
        self.obj = template.Variable(obj)
        self.kwargs = kwargs

    def render(self, context):
        obj = self.obj.resolve(context)
        value = ratehandler.get_object_rate(obj, **self.kwargs)
        if value == None:
            return self.kwargs.get('default', '')
        else:
            return value

def do_krate(parser, token):
    splited = token.split_contents()
    if len(splited) == 1:
        raise template.TemplateSyntaxError("krate tag requires one or more arguments")
    else:
        obj = splited[1]
        tail = splited[2:]

    kwargs = {}
    for pair in tail:
        splited_pair = pair.split("=")
        if len(splited_pair) != 2:
            raise template.TemplateSyntaxError("krate tag kwargs must be key=value format")
        kwargs[splited_pair[0]] = splited_pair[1]

    return KRateNode(obj, **kwargs)

class MyKRateNode(template.Node):
    def __init__(self, obj, request_or_user, **kwargs):
        self.obj = template.Variable(obj)
        self.request_or_user = template.Variable(request_or_user)
        self.kwargs = kwargs

    def render(self, context):
        obj = self.obj.resolve(context)
        request_or_user = self.request_or_user.resolve(context)

        if isinstance(request_or_user, HttpRequest):
            value = ratehandler.get_request_object_rate(request_or_user, obj, **self.kwargs)
        elif isinstance(request_or_user, User):
            value = ratehandler.get_user_object_rate(request_or_user, obj, **self.kwargs)
        elif isinstance(request_or_user, AnonymousUser):
            raise template.TemplateSyntaxError("mykrate second argument can't be the anonymous user")
        else:
            raise template.TemplateSyntaxError("mykrate second argument must be an resquest or user")

        if value == None:
            return self.kwargs.get('default', '')
        else:
            return value


def do_mykrate(parser, token):
    splited = token.split_contents()
    if len(splited) < 3:
        raise template.TemplateSyntaxError("mykrate tag requires two or more arguments")
    else:
        obj = splited[1]
        request_or_user = splited[2]
        tail = splited[3:]

    kwargs = {}
    for pair in tail:
        splited_pair = pair.split("=")
        if len(splited_pair) != 2 or len(splited_pair[0]) == 0:
            raise template.TemplateSyntaxError("mykrate tag kwargs must be key=value format")
        kwargs[splited_pair[0]] = splited_pair[1]

    return MyKRateNode(obj, request_or_user, **kwargs)

register.tag('krate', do_krate)
register.tag('mykrate', do_mykrate)



from django.db import models
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic


class ObjRate(models.Model):
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    krate = models.FloatField(null=True, blank=True, default=None)
    user = models.ForeignKey(User, null=False, blank=False)

class ObjRateAggregate(models.Model):
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    summatory = models.FloatField(null=False, blank=False, default=0)
    count = models.IntegerField(null=False, blank=False, default=0)
    avg = models.FloatField(null=True, blank=True, default=None)

class KRateableMixin(models.Model):
    krate = models.FloatField(null=True, blank=True, default=None)

    class Meta:
        abstract = True



from django.db import transaction
from ..ratehandlerbase import RateHandlerBase
from .models import *

class DBRateHandler(RateHandlerBase):
    def rate_object(self, request, obj, rate, **kwargs):
        with transaction.commit_on_success():
            try:
                rate = float(rate)
            except ValueError:
                return ('invalid-rate-format', rate)
            obj_content_type = ContentType.objects.get_for_model(obj)
            (obj_rate, created) = ObjRate.objects.get_or_create(content_type=obj_content_type, object_id=obj.pk, user=request.user)
            (obj_rate_aggregate, created_agg) = ObjRateAggregate.objects.get_or_create(content_type=obj_content_type, object_id=obj.pk)

            if created_agg:
                obj_rate.krate = rate
                obj_rate.save()
                obj_rate_aggregate.count = 1
                obj_rate_aggregate.summatory = rate
                obj_rate_aggregate.avg = rate
                obj_rate_aggregate.save()
                if isinstance(obj, KRateableMixin):
                    obj.krate = rate
                    obj.save()
                return ('created', rate)
            elif not created_agg and created:
                obj_rate.krate = rate
                obj_rate.save()
                obj_rate_aggregate.count += 1
                obj_rate_aggregate.summatory += rate
                obj_rate_aggregate.avg = obj_rate_aggregate.summatory / obj_rate_aggregate.count
                obj_rate_aggregate.save()
                if isinstance(obj, KRateableMixin):
                    obj.krate = obj_rate_aggregate.avg
                    obj.save()
                return ('created', obj_rate_aggregate.avg)
            elif not created_agg and not created:
                obj_rate_aggregate.summatory -= obj_rate.krate
                obj_rate_aggregate.summatory += rate
                obj_rate_aggregate.avg = obj_rate_aggregate.summatory / obj_rate_aggregate.count
                obj_rate_aggregate.save()
                obj_rate.krate = rate
                obj_rate.save()
                if isinstance(obj, KRateableMixin):
                    obj.krate = obj_rate_aggregate.avg
                    obj.save()
                return ('modified', obj_rate_aggregate.avg)

    def get_object_rate(self, obj, **kwargs):
        if isinstance(obj, KRateableMixin):
            return obj.krate
        else:
            try:
                obj_content_type = ContentType.objects.get_for_model(obj)
                obj_rate = ObjRateAggregate.objects.get(content_type=obj_content_type, object_id=obj.pk)
                return obj_rate.avg
            except ObjVisit.DoesNotExist:
                return None

    def get_request_object_rate(self, request, obj, **kwargs):
        return self.get_user_object_rate(request.user, obj, **kwargs)

    def get_user_object_rate(self, user, obj, **kwargs):
        try:
            obj_content_type = ContentType.objects.get_for_model(obj)
            obj_rate = ObjRate.objects.get(content_type=obj_content_type, object_id=obj.pk, user=user)
            return obj_rate.krate
        except ObjRate.DoesNotExist:
            return None



from . import settings
from .ratehandlers import ratehandler

def rate_object(request, obj, rate, **kwargs):
    return ratehandler.rate_object(request, obj, rate, **kwargs)

def get_object_rate(obj, **kwargs):
    return ratehandler.get_object_rate(obj, **kwargs)

def get_request_object_rate(request, obj, **kwargs):
    return ratehandler.get_request_object_rate(request, obj, **kwargs)

def get_user_object_rate(user, obj, **kwargs):
    return ratehandler.get_user_object_rate(user, obj, **kwargs)



import unittest

from django.http import HttpRequest
from django.contrib.auth.models import User

from krate.ratehandlers.noratehandler import NoRateHandler
from krate.ratehandlers.dbratehandler import DBRateHandler
from krate.ratehandlers.dbratehandler.models import *

from testing.models import TestModel


class NoRateHandlerTest(unittest.TestCase):
    def setUp(self):
        self.handler = NoRateHandler()
        self.request = HttpRequest()
        self.request.META = {
            'CONTENT_LENGTH': '',
            'CONTENT_TYPE': 'text/html',
            'HTTP_ACCEPT_ENCODING': 'utf-8',
            'HTTP_ACCEPT_LANGUAGE': 'es',
            'HTTP_HOST': 'testhost',
            'HTTP_REFERER': 'testreferer',
            'HTTP_USER_AGENT': 'test user agent',
            'QUERY_STRING': '/',
            'REMOTE_ADDR': 'testaddress',
            'REMOTE_HOST': 'testhost',
            'REMOTE_USER': 'testuser',
            'REQUEST_METHOD': 'GET',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': '80',
        }
        (self.request.user, created) = User.objects.get_or_create(username='testuser')
        self.request.user.save()
        self.test1 = TestModel()
        self.test1.save()

    def test_rate_object(self):
        self.assertEqual(self.handler.rate_object(self.request, self.test1, 10), ('created', 0))

    def test_get_object_rate(self):
        self.assertEqual(self.handler.get_object_rate(self.test1), 0)

    def test_get_request_object_rate(self):
        self.assertEqual(self.handler.get_request_object_rate(self.request, self.test1), 0)

    def test_get_user_object_rate(self):
        self.assertEqual(self.handler.get_user_object_rate(self.request.user, self.test1), 0)


class DBRateHandlerTest(unittest.TestCase):
    def setUp(self):
        self.handler = DBRateHandler()
        self.request1 = HttpRequest()
        self.request1.META = {
            'CONTENT_LENGTH': '',
            'CONTENT_TYPE': 'text/html',
            'HTTP_ACCEPT_ENCODING': 'utf-8',
            'HTTP_ACCEPT_LANGUAGE': 'es',
            'HTTP_HOST': 'testhost',
            'HTTP_REFERER': 'testreferer',
            'HTTP_USER_AGENT': 'test user agent',
            'QUERY_STRING': '/',
            'REMOTE_ADDR': 'testaddress',
            'REMOTE_HOST': 'testhost',
            'REMOTE_USER': 'testuser',
            'REQUEST_METHOD': 'GET',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': '80',
        }
        (self.request1.user, created) = User.objects.get_or_create(username='testuser1')
        self.request1.user.save()

        self.request2 = HttpRequest()
        self.request2.META = {
            'CONTENT_LENGTH': '',
            'CONTENT_TYPE': 'text/html',
            'HTTP_ACCEPT_ENCODING': 'utf-8',
            'HTTP_ACCEPT_LANGUAGE': 'es',
            'HTTP_HOST': 'testhost',
            'HTTP_REFERER': 'testreferer',
            'HTTP_USER_AGENT': 'test user agent',
            'QUERY_STRING': '/',
            'REMOTE_ADDR': 'testaddress',
            'REMOTE_HOST': 'testhost',
            'REMOTE_USER': 'testuser',
            'REQUEST_METHOD': 'GET',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': '80',
        }
        (self.request2.user, created) = User.objects.get_or_create(username='testuser2')
        self.request2.user.save()
        self.test1 = TestModel()
        self.test1.save()
        self.test2 = TestModel()
        self.test2.save()

    def test_rate_object(self):
        self.assertEqual(self.handler.rate_object(self.request1, self.test1, 10), ('created', 10))
        self.assertEqual(self.handler.rate_object(self.request1, self.test1, 5), ('modified', 5))
        self.assertEqual(self.handler.rate_object(self.request2, self.test1, 10), ('created', 7.5))
        self.assertEqual(self.handler.rate_object(self.request2, self.test1, 3), ('modified', 4))

        self.assertEqual(self.handler.rate_object(self.request1, self.test2, 10), ('created', 10))
        self.assertEqual(self.handler.rate_object(self.request1, self.test2, 5), ('modified', 5))
        self.assertEqual(self.handler.rate_object(self.request2, self.test2, 10), ('created', 7.5))
        self.assertEqual(self.handler.rate_object(self.request2, self.test2, 3), ('modified', 4))

    def test_get_object_rate(self):
        self.assertEqual(self.handler.rate_object(self.request1, self.test1, 10), ('created', 10))
        self.assertEqual(self.handler.get_object_rate(self.test1), 10)
        self.assertEqual(self.handler.rate_object(self.request1, self.test1, 5), ('modified', 5))
        self.assertEqual(self.handler.get_object_rate(self.test1), 5)
        self.assertEqual(self.handler.rate_object(self.request2, self.test1, 10), ('created', 7.5))
        self.assertEqual(self.handler.get_object_rate(self.test1), 7.5)
        self.assertEqual(self.handler.rate_object(self.request2, self.test1, 3), ('modified', 4))
        self.assertEqual(self.handler.get_object_rate(self.test1), 4)

        self.assertEqual(self.handler.rate_object(self.request1, self.test2, 10), ('created', 10))
        self.assertEqual(self.handler.get_object_rate(self.test2), 10)
        self.assertEqual(self.handler.rate_object(self.request1, self.test2, 5), ('modified', 5))
        self.assertEqual(self.handler.get_object_rate(self.test2), 5)
        self.assertEqual(self.handler.rate_object(self.request2, self.test2, 10), ('created', 7.5))
        self.assertEqual(self.handler.get_object_rate(self.test2), 7.5)
        self.assertEqual(self.handler.rate_object(self.request2, self.test2, 3), ('modified', 4))
        self.assertEqual(self.handler.get_object_rate(self.test2), 4)

    def test_get_request_object_rate(self):
        self.assertEqual(self.handler.rate_object(self.request1, self.test1, 10), ('created', 10))
        self.assertEqual(self.handler.get_request_object_rate(self.request1, self.test1), 10)
        self.assertEqual(self.handler.rate_object(self.request1, self.test1, 5), ('modified', 5))
        self.assertEqual(self.handler.get_request_object_rate(self.request1, self.test1), 5)

        self.assertEqual(self.handler.rate_object(self.request2, self.test1, 10), ('created', 7.5))
        self.assertEqual(self.handler.get_request_object_rate(self.request2, self.test1), 10)
        self.assertEqual(self.handler.rate_object(self.request2, self.test1, 3), ('modified', 4))
        self.assertEqual(self.handler.get_request_object_rate(self.request2, self.test1), 3)

        self.assertEqual(self.handler.rate_object(self.request1, self.test2, 10), ('created', 10))
        self.assertEqual(self.handler.get_request_object_rate(self.request1, self.test2), 10)
        self.assertEqual(self.handler.rate_object(self.request1, self.test2, 5), ('modified', 5))
        self.assertEqual(self.handler.get_request_object_rate(self.request1, self.test2), 5)

        self.assertEqual(self.handler.rate_object(self.request2, self.test2, 10), ('created', 7.5))
        self.assertEqual(self.handler.get_request_object_rate(self.request2, self.test2), 10)
        self.assertEqual(self.handler.rate_object(self.request2, self.test2, 3), ('modified', 4))
        self.assertEqual(self.handler.get_request_object_rate(self.request2, self.test2), 3)

    def test_get_user_object_rate(self):
        self.assertEqual(self.handler.rate_object(self.request1, self.test1, 10), ('created', 10))
        self.assertEqual(self.handler.get_user_object_rate(self.request1.user, self.test1), 10)
        self.assertEqual(self.handler.rate_object(self.request1, self.test1, 5), ('modified', 5))
        self.assertEqual(self.handler.get_user_object_rate(self.request1.user, self.test1), 5)

        self.assertEqual(self.handler.rate_object(self.request2, self.test1, 10), ('created', 7.5))
        self.assertEqual(self.handler.get_user_object_rate(self.request2.user, self.test1), 10)
        self.assertEqual(self.handler.rate_object(self.request2, self.test1, 3), ('modified', 4))
        self.assertEqual(self.handler.get_user_object_rate(self.request2.user, self.test1), 3)

        self.assertEqual(self.handler.rate_object(self.request1, self.test2, 10), ('created', 10))
        self.assertEqual(self.handler.get_user_object_rate(self.request1.user, self.test2), 10)
        self.assertEqual(self.handler.rate_object(self.request1, self.test2, 5), ('modified', 5))
        self.assertEqual(self.handler.get_user_object_rate(self.request1.user, self.test2), 5)

        self.assertEqual(self.handler.rate_object(self.request2, self.test2, 10), ('created', 7.5))
        self.assertEqual(self.handler.get_user_object_rate(self.request2.user, self.test2), 10)
        self.assertEqual(self.handler.rate_object(self.request2, self.test2, 3), ('modified', 4))
        self.assertEqual(self.handler.get_user_object_rate(self.request2.user, self.test2), 3)
